@@ -14,7 +14,7 @@ use strict;
 use vars qw($VERSION %csType);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.07';
+$VERSION = '1.11';
 
 my %charsetTable;   # character set tables we've loaded
 
@@ -57,6 +57,8 @@ my %unicode2byte = (
     Hebrew       => 0x101,
     Latin        => 0x101,
     Latin2       => 0x101,
+    DOSLatinUS   => 0x101,
+    DOSLatin1    => 0x101,
     MacCroatian  => 0x101,
     MacCyrillic  => 0x101,
     MacGreek     => 0x101,
@@ -105,6 +107,33 @@ sub LoadCharset($)
     return $conv;
 }
 
+#------------------------------------------------------------------------------
+# Does an array contain valid UTF-16 characters?
+# Inputs: 0) array reference to list of UCS-2 values
+# Returns: 0=invalid UTF-16, 1=valid UTF-16 with no surrogates, 2=valid UTF-16 with surrogates
+sub IsUTF16($)
+{
+    local $_;
+    my $uni = shift;
+    my $surrogate;
+    foreach (@$uni) {
+        my $hiBits = ($_ & 0xfc00);
+        if ($hiBits == 0xfc00) {
+            # check for invalid values in UTF-16
+            return 0 if $_ == 0xffff or $_ == 0xfffe or ($_ >= 0xfdd0 and $_ <= 0xfdef);
+        } elsif ($surrogate) {
+            return 0 if $hiBits != 0xdc00;
+            $surrogate = 0;
+        } else {
+            return 0 if $hiBits == 0xdc00;
+            $surrogate = 1 if $hiBits == 0xd800;
+        }
+    }
+    return 1 if not defined $surrogate;
+    return 2 unless $surrogate;
+    return 0;
+}
+
 #------------------------------------------------------------------------------
 # Decompose string with specified encoding into an array of integer code points
 # Inputs: 0) ExifTool object ref (or undef), 1) string, 2) character set name,
@@ -114,10 +143,11 @@ sub LoadCharset($)
 # - byte order only used for fixed-width 2-byte and 4-byte character sets
 # - byte order mark observed and then removed with UCS2 and UCS4
 # - no warnings are issued if ExifTool object is not provided
+# - sets ExifTool WrongByteOrder flag if byte order is Unknown and current order is wrong
 sub Decompose($$$;$)
 {
     local $_;
-    my ($exifTool, $val, $charset) = @_; # ($byteOrder assigned later if required)
+    my ($et, $val, $charset) = @_; # ($byteOrder assigned later if required)
     my $type = $csType{$charset};
     my (@uni, $conv);
 
@@ -125,7 +155,7 @@ sub Decompose($$$;$)
         $conv = LoadCharset($charset);
         unless ($conv) {
             # (shouldn't happen)
-            $exifTool->Warn("Invalid character set $charset") if $exifTool;
+            $et->Warn("Invalid character set $charset") if $et;
             return \@uni;   # error!
         }
     } elsif ($type == 0x100) {
@@ -140,9 +170,9 @@ sub Decompose($$$;$)
             # (somehow the meaning of "U0" was reversed in Perl 5.10.0!)
             @uni = unpack($] < 5.010000 ? 'U0U*' : 'C0U*', $val);
             # issue warning if we had errors
-            if ($Image::ExifTool::evalWarning and $exifTool and not $$exifTool{WarnBadUTF8}) {
-                $exifTool->Warn('Malformed UTF-8 character(s)');
-                $$exifTool{WarnBadUTF8} = 1;
+            if ($Image::ExifTool::evalWarning and $et and not $$et{WarnBadUTF8}) {
+                $et->Warn('Malformed UTF-8 character(s)');
+                $$et{WarnBadUTF8} = 1;
             }
         }
         return \@uni;       # all done!
@@ -195,6 +225,7 @@ sub Decompose($$$;$)
                     # we guessed wrong, so decode using the other byte order
                     $fmt =~ tr/nvNV/vnVN/;
                     @uni = unpack($fmt, $val);
+                    $$et{WrongByteOrder} = 1;
                 }
             }
             # handle surrogate pairs of UTF-16
@@ -224,7 +255,10 @@ sub Decompose($$$;$)
                     ++$e2;
                 }
                 # use this byte order if there are fewer errors
-                return \@try if $e2 < $e1;
+                if ($e2 < $e1) {
+                    $$et{WrongByteOrder} = 1;
+                    return \@try;
+                }
             }
         } else {
             # translate any characters found in the lookup
@@ -274,9 +308,9 @@ sub Decompose($$$;$)
 sub Recompose($$;$$)
 {
     local $_;
-    my ($exifTool, $uni, $charset) = @_; # ($byteOrder assigned later if required)
+    my ($et, $uni, $charset) = @_; # ($byteOrder assigned later if required)
     my ($outVal, $conv, $inv);
-    $charset or $charset = $$exifTool{OPTIONS}{Charset};
+    $charset or $charset = $$et{OPTIONS}{Charset};
     my $csType = $csType{$charset};
     if ($csType == 0x100) {     # UTF8 (also treat ASCII as UTF8)
         if ($] >= 5.006001) {
@@ -293,14 +327,14 @@ sub Recompose($$;$$)
     if ($csType & 0x801) {
         $conv = LoadCharset($charset);
         unless ($conv) {
-            $exifTool->Warn("Missing charset $charset") if $exifTool;
+            $et->Warn("Missing charset $charset") if $et;
             return '';
         }
         $inv = $unicode2byte{$charset};
         # generate inverse lookup if necessary
         unless ($inv) {
             if (not $csType or $csType & 0x802) {
-                $exifTool->Warn("Invalid destination charset $charset") if $exifTool;
+                $et->Warn("Invalid destination charset $charset") if $et;
                 return '';
             }
             # prepare table to convert from Unicode to 1-byte characters
@@ -321,9 +355,9 @@ sub Recompose($$;$$)
             # different character with this byte value
             next if $_ < 0x100 and not $$conv{$_};
             $_ = ord('?');  # set invalid characters to '?'
-            if ($exifTool and not $$exifTool{EncodingError}) {
-                $exifTool->Warn("Some character(s) could not be encoded in $charset");
-                $$exifTool{EncodingError} = 1;
+            if ($et and not $$et{EncodingError}) {
+                $et->Warn("Some character(s) could not be encoded in $charset");
+                $$et{EncodingError} = 1;
             }
         }
         # repack as an 8-bit string and truncate at null
@@ -373,10 +407,11 @@ This module contains routines used by ExifTool to translate special
 character sets.  Currently, the following character sets are supported:
 
   UTF8, UTF16, UCS2, UCS4, Arabic, Baltic, Cyrillic, Greek, Hebrew, JIS,
-  Latin, Latin2, MacArabic, MacChineseCN, MacChineseTW, MacCroatian,
-  MacCyrillic, MacGreek, MacHebrew, MacIceland, MacJapanese, MacKorean,
-  MacLatin2, MacRSymbol, MacRoman, MacRomanian, MacThai, MacTurkish,
-  PDFDoc, RSymbol, ShiftJIS, Symbol, Thai, Turkish, Vietnam
+  Latin, Latin2, DOSLatinUS, DOSLatin1, MacArabic, MacChineseCN,
+  MacChineseTW, MacCroatian, MacCyrillic, MacGreek, MacHebrew, MacIceland,
+  MacJapanese, MacKorean, MacLatin2, MacRSymbol, MacRoman, MacRomanian,
+  MacThai, MacTurkish, PDFDoc, RSymbol, ShiftJIS, Symbol, Thai, Turkish,
+  Vietnam
 
 However, only some of these character sets are available to the user via
 ExifTool options -- the multi-byte character sets are used only internally
@@ -384,7 +419,7 @@ when decoding certain types of information.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
