@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD %iptcCharset);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.52';
+$VERSION = '1.57';
 
 %iptcCharset = (
     "\x1b%G"  => 'UTF8',
@@ -202,7 +202,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     80 => {
         Name => 'TimeSent',
@@ -211,7 +211,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     90 => {
         Name => 'CodedCharacterSet',
@@ -228,6 +228,7 @@ my %fileFormat = (
         },
         Protected => 1,
         Format => 'string[0,32]',
+        ValueConvInv => '$val =~ /^UTF-?8$/i ? "\x1b%G" : $val',
         # convert ISO 2022 escape sequences to a more readable format
         PrintConv => \&PrintCodedCharset,
         PrintConvInv => \&PrintInvCodedCharset,
@@ -339,7 +340,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     35 => {
         Name => 'ReleaseTime',
@@ -348,7 +349,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     37 => {
         Name => 'ExpirationDate',
@@ -357,7 +358,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     38 => {
         Name => 'ExpirationTime',
@@ -366,7 +367,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     40 => {
         Name => 'SpecialInstructions',
@@ -396,7 +397,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     50 => {
         Name => 'ReferenceNumber',
@@ -410,7 +411,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     60 => {
         Name => 'TimeCreated',
@@ -419,7 +420,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     62 => {
         Name => 'DigitalCreationDate',
@@ -428,7 +429,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     63 => {
         Name => 'DigitalCreationTime',
@@ -437,7 +438,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     65 => {
         Name => 'OriginatingProgram',
@@ -533,7 +534,7 @@ my %fileFormat = (
             I haven't found a reference for the format of tags 121, 184-188 and
             225-232, so I have just make them writable as strings with
             reasonable length.  Beware that if this is wrong, other utilities
-            won't be able to read these tags as written by ExifTool
+            may not be able to read these tags as written by ExifTool
         },
     },
     122 => {
@@ -638,7 +639,7 @@ my %fileFormat = (
     },
     202 => {
         Name => 'ObjectPreviewData',
-        Groups => { 2 => 'Image' },
+        Groups => { 2 => 'Preview' },
         Format => 'undef[0,256000]',
         Binary => 1,
     },
@@ -673,6 +674,8 @@ my %fileFormat = (
     231 => {
         Name => 'DocumentHistory',
         Format => 'string[0,256]', # (guess)
+        ValueConv => '$val =~ s/\0+/\n/g; $val', # (have seen embedded nulls)
+        ValueConvInv => '$val',
     },
     232 => {
         Name => 'ExifCameraInfo',
@@ -1047,6 +1050,7 @@ sub ProcessIPTC($$$)
     my $dirLen = $$dirInfo{DirLen} || 0;
     my $dirEnd = $pos + $dirLen;
     my $verbose = $et->Options('Verbose');
+    my $validate = $et->Options('Validate');
     my $success = 0;
     my ($lastRec, $recordPtr, $recordName);
 
@@ -1073,11 +1077,18 @@ sub ProcessIPTC($$$)
                 }
                 $et->FoundTag('CurrentIPTCDigest', $md5);
             }
-        } elsif ($Image::ExifTool::MWG::strict and $$et{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/) {
-            # ignore non-standard IPTC while in strict MWG compatibility mode
-            $et->Warn("Ignored non-standard IPTC at $path");
-            return 1;
         } else {
+            if (($Image::ExifTool::MWG::strict or $et->Options('Validate')) and
+                $$et{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/)
+            {
+                if ($Image::ExifTool::MWG::strict) {
+                    # ignore non-standard IPTC while in strict MWG compatibility mode
+                    $et->Warn("Ignored non-standard IPTC at $path");
+                    return 1;
+                } else {
+                    $et->Warn("Non-standard IPTC at $path", 1);
+                }
+            }
             # extract non-standard IPTC
             my $count = ($$et{DIR_COUNT}{IPTC} || 0) + 1;  # count non-standard IPTC
             $$et{DIR_COUNT}{IPTC} = $count;
@@ -1143,6 +1154,9 @@ sub ProcessIPTC($$$)
             last;
         }
         if (not defined $lastRec or $lastRec != $rec) {
+            if ($validate and defined $lastRec and $rec < $lastRec) {
+                $et->Warn("IPTC doesn't conform to spec: Records out of sequence",1)
+            }
             my $tableInfo = $tagTablePtr->{$rec};
             unless ($tableInfo) {
                 $et->WarnOnce("Unrecognized IPTC record $rec (ignored)");
@@ -1173,9 +1187,26 @@ sub ProcessIPTC($$$)
         # (could use $$recordPtr{FORMAT} if no Format below, but don't do this to
         #  be backward compatible with improperly written PhotoMechanic tags)
         $format = $$tagInfo{Format} if $tagInfo;
-        # use logic to determine format if not specified
-        unless ($format) {
+        if (not $format) {
+            # guess at "int" format if not specified
             $format = 'int' if $len <= 4 and $len != 3 and $val =~ /[\0-\x08]/;
+        } elsif ($validate) {
+            my ($fmt,$min,$max);
+            if ($format =~ /(.*)\[(\d+)(,(\d+))?\]/) {
+                $fmt = $1;
+                $min = $2;
+                $max = $4 || $2;
+            } else {
+                $fmt = $format;
+                $min = $max = 1;
+            }
+            my $siz = Image::ExifTool::FormatSize($fmt) || 1;
+            $min *= $siz; $max *= $siz;
+            if ($len < $min or $len > $max) {
+                my $should = ($min == $max) ? $min : ($len < $min ? "$min min" : "$max max");
+                my $what = ($len < $siz * $min) ? 'short' : 'long';
+                $et->Warn("IPTC $$tagInfo{Name} too $what ($len bytes; should be $should)", 1);
+            }
         }
         if ($format) {
             if ($format =~ /^int/) {
@@ -1187,7 +1218,10 @@ sub ProcessIPTC($$$)
                     }
                 }
             } elsif ($format =~ /^string/) {
-                $val =~ s/\0+$//;   # some braindead softwares add null terminators
+                # some braindead softwares add null terminators
+                if ($val =~ s/\0+$// and $validate) {
+                    $et->Warn("IPTC $$tagInfo{Name} improperly terminated", 1);
+                }
                 if ($rec == 1) {
                     # handle CodedCharacterSet tag
                     $xlat = HandleCodedCharset($et, $val) if $tag == 90;
@@ -1197,9 +1231,11 @@ sub ProcessIPTC($$$)
                     TranslateCodedString($et, \$val, \$xlat, 1);
                 }
             } elsif ($format =~ /^digits/) {
-                $val =~ s/\0+$//;
+                if ($val =~ s/\0+$// and $validate) {
+                    $et->Warn("IPTC $$tagInfo{Name} improperly terminated", 1);
+                }
             } elsif ($format !~ /^undef/) {
-                warn("Invalid IPTC format: $format");
+                warn("Invalid IPTC format: $format");   # (this would be a programming error)
             }
         }
         $verbose and $et->VerboseInfo($tag, $tagInfo,
@@ -1243,7 +1279,7 @@ image files.
 
 =head1 AUTHOR
 
-Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
