@@ -7,6 +7,9 @@
 #
 # References:   1) http://www.mp3-tech.org/
 #               2) http://www.getid3.org/
+#               3) http://dvd.sourceforge.net/dvdinfo/dvdmpeg.html
+#               4) http://ffmpeg.org/
+#               5) http://sourceforge.net/projects/mediainfo/
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::MPEG;
@@ -15,7 +18,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.08';
+$VERSION = '1.16';
 
 %Image::ExifTool::MPEG::Audio = (
     GROUPS => { 2 => 'Audio' },
@@ -43,7 +46,8 @@ $VERSION = '1.08';
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} == 3 and $self->{MPEG_Layer} == 3',
             Notes => 'version 1, layer 1',
-            PrintConv => {
+            PrintConvColumns => 3,
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 64000,
@@ -60,12 +64,14 @@ $VERSION = '1.08';
                 13 => 416000,
                 14 => 448000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} == 3 and $self->{MPEG_Layer} == 2',
             Notes => 'version 1, layer 2',
-            PrintConv => {
+            PrintConvColumns => 3,
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 48000,
@@ -82,12 +88,14 @@ $VERSION = '1.08';
                 13 => 320000,
                 14 => 384000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} == 3 and $self->{MPEG_Layer} == 1',
             Notes => 'version 1, layer 3',
-            PrintConv => {
+            PrintConvColumns => 3,
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 40000,
@@ -104,12 +112,14 @@ $VERSION = '1.08';
                 13 => 256000,
                 14 => 320000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} != 3 and $self->{MPEG_Layer} == 3',
             Notes => 'version 2 or 2.5, layer 1',
-            PrintConv => {
+            PrintConvColumns => 3,
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 48000,
@@ -126,12 +136,14 @@ $VERSION = '1.08';
                 13 => 224000,
                 14 => 256000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} != 3 and $self->{MPEG_Layer}',
             Notes => 'version 2 or 2.5, layer 2 or 3',
-            PrintConv => {
+            PrintConvColumns => 3,
+            ValueConv => {
                 0 => 'free',
                 1 => 8000,
                 2 => 16000,
@@ -148,6 +160,7 @@ $VERSION = '1.08';
                 13 => 144000,
                 14 => 160000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
     ],
     'Bit20-21' => [
@@ -298,6 +311,7 @@ $VERSION = '1.08';
     'Bit32-49' => {
         Name => 'VideoBitrate',
         ValueConv => '$val eq 0x3ffff ? "Variable" : $val * 400',
+        PrintConv => 'ConvertBitrate($val)',
     },
     # these tags not very interesting
     #'Bit50'    => 'MarkerBit',
@@ -306,13 +320,65 @@ $VERSION = '1.08';
     #'Bit62'    => 'IntraQuantMatrixFlag',
 );
 
-%Image::ExifTool::MPEG::VBR = (
+%Image::ExifTool::MPEG::Xing = (
     GROUPS => { 2 => 'Audio' },
     VARS => { NO_ID => 1 },
-    NOTES => 'These tags are extracted for variable bitrate audio.',
+    NOTES => 'These tags are extracted from the Xing/Info frame.',
     1 => { Name => 'VBRFrames' },
     2 => { Name => 'VBRBytes' },
     3 => { Name => 'VBRScale' },
+    4 => { Name => 'Encoder' },
+    5 => { Name => 'LameVBRQuality' },
+    6 => { Name => 'LameQuality' },
+    7 => { # (for documentation only)
+        Name => 'LameHeader',
+        SubDirectory => { TagTable => 'Image::ExifTool::MPEG::Lame' },
+    },
+);
+
+# Lame header tags (ref 5)
+%Image::ExifTool::MPEG::Lame = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Audio' },
+    NOTES => 'Tags extracted from Lame 3.90 or later header.',
+    9 => {
+        Name => 'LameMethod',
+        Mask => 0x0f,
+        PrintConv => {
+            1 => 'CBR',
+            2 => 'ABR',
+            3 => 'VBR (old/rh)',
+            4 => 'VBR (new/mtrh)',
+            5 => 'VBR (old/rh)',
+            6 => 'VBR',
+            8 => 'CBR (2-pass)',
+            9 => 'ABR (2-pass)',
+        },
+    },
+    10 => {
+        Name => 'LameLowPassFilter',
+        ValueConv => '$val * 100',
+        PrintConv => '($val / 1000) . " kHz"',
+    },
+    # 19 - EncodingFlags
+    20 => {
+        Name => 'LameBitrate',
+        ValueConv => '$val * 1000',
+        PrintConv => 'ConvertBitrate($val)',
+    },
+    24 => {
+        Name => 'LameStereoMode',
+        Mask => 0x1c,
+        PrintConv => {
+            0 => 'Mono',
+            1 => 'Stereo',
+            2 => 'Dual Channels',
+            3 => 'Joint Stereo',
+            4 => 'Forced Joint Stereo',
+            6 => 'Auto',
+            7 => 'Intensity Stereo',
+        },
+    },
 );
 
 # composite tags
@@ -330,6 +396,7 @@ $VERSION = '1.08';
             5 => 'MPEG:SampleRate',
             6 => 'MPEG:MPEGAudioVersion',
         },
+        Priority => -1, # (don't want to replace any other Duration tag)
         ValueConv => q{
             if ($val[4] and defined $val[5] and defined $val[6]) {
                 # calculate from number of VBR audio frames
@@ -340,12 +407,28 @@ $VERSION = '1.08';
             # calculate duration as file size divided by total bitrate
             # (note: this is only approximate!)
             return undef unless $val[2] or $val[3];
-            return undef if $prt[2] and not $prt[2] =~ /^\d+$/;
+            return undef if $val[2] and not $val[2] =~ /^\d+$/;
             return undef if $val[3] and not $val[3] =~ /^\d+$/;
-            return (8 * ($val[0] - ($val[1]||0))) / (($prt[2]||0) + ($val[3]||0));
+            return (8 * ($val[0] - ($val[1]||0))) / (($val[2]||0) + ($val[3]||0));
         },
         PrintConv => 'ConvertDuration($val) . " (approx)"',
     },
+    AudioBitrate => {
+        Groups => { 2 => 'Audio' },
+        Notes => 'calculated for variable-bitrate MPEG audio',
+        Require => {
+            0 => 'MPEG:MPEGAudioVersion',
+            1 => 'MPEG:SampleRate',
+            2 => 'MPEG:VBRBytes',
+            3 => 'MPEG:VBRFrames',
+        },
+        ValueConv => q{
+            return undef unless $val[3];
+            my $mfs = $prt[1] / ($val[0] == 3 ? 144 : 72);
+            return $mfs * $val[2] / $val[3];
+        },
+        PrintConv => 'ConvertBitrate($val)',
+    },
 );
 
 # add our composite tags
@@ -357,7 +440,7 @@ Image::ExifTool::AddCompositeTags('Image::ExifTool::MPEG');
 # Inputs: 0) ExifTool object ref, 1) tag table ref, 2-N) list of 32-bit data words
 sub ProcessFrameHeader($$@)
 {
-    my ($exifTool, $tagTablePtr, @data) = @_;
+    my ($et, $tagTablePtr, @data) = @_;
     my $tag;
     foreach $tag (sort keys %$tagTablePtr) {
         next unless $tag =~ /^Bit(\d{2})-?(\d{2})?/;
@@ -369,19 +452,20 @@ sub ProcessFrameHeader($$@)
             $mask += (1 << $_);
         }
         my $val = ($word >> (31 + 32*$index - $b2)) & $mask;
-        $exifTool->HandleTag($tagTablePtr, $tag, $val);
+        $et->HandleTag($tagTablePtr, $tag, $val);
     }
 }
 
 #------------------------------------------------------------------------------
 # Read MPEG audio frame header
 # Inputs: 0) ExifTool object reference, 1) Reference to audio data
+#         2) flag set if we are trying to recognized MP3 file only
 # Returns: 1 on success, 0 if no audio header was found
-sub ParseMPEGAudio($$)
+sub ParseMPEGAudio($$;$)
 {
-    my ($exifTool, $buffPt) = @_;
+    my ($et, $buffPt, $mp3) = @_;
     my ($word, $pos);
-    my $ext = $$exifTool{FILE_EXT} || '';
+    my $ext = $$et{FILE_EXT} || '';
 
     for (;;) {
         # find frame sync
@@ -397,7 +481,8 @@ sub ParseMPEGAudio($$)
             ($word & 0x00f000) == 0x000000 or   # 0000 is the "free" bitrate index
             ($word & 0x00f000) == 0x00f000 or   # 1111 is a bad bitrate index
             ($word & 0x000c00) == 0x000c00 or   # 11 is a reserved sampling frequency
-            ($word & 0x000003) == 0x000002)     # 10 is a reserved emphasis
+            ($word & 0x000003) == 0x000002 or   # 10 is a reserved emphasis
+            (($mp3 and ($word & 0x060000) != 0x020000))) # must be layer 3 for MP3
         {
             # give up easily unless this really should be an MP3 file
             return 0 unless $ext eq 'MP3';
@@ -408,30 +493,32 @@ sub ParseMPEGAudio($$)
         last;
     }
     # set file type if not done already
-    $exifTool->SetFileType() unless $exifTool->{VALUE}->{FileType};
+    $et->SetFileType();
 
     my $tagTablePtr = GetTagTable('Image::ExifTool::MPEG::Audio');
-    ProcessFrameHeader($exifTool, $tagTablePtr, $word);
+    ProcessFrameHeader($et, $tagTablePtr, $word);
 
     # extract the VBR information (ref MP3::Info)
-    my ($v, $m) = ($$exifTool{MPEG_Vers}, $$exifTool{MPEG_Mode});
+    my ($v, $m) = ($$et{MPEG_Vers}, $$et{MPEG_Mode});
     while (defined $v and defined $m) {
         my $len = length $$buffPt;
         $pos += $v == 3 ? ($m == 3 ? 17 : 32) : ($m == 3 ?  9 : 17);
         last if $pos + 8 > $len;
         my $buff = substr($$buffPt, $pos, 8);
         last unless $buff =~ /^(Xing|Info)/;
-        my $vbrTable = GetTagTable('Image::ExifTool::MPEG::VBR');
+        my $xingTable = GetTagTable('Image::ExifTool::MPEG::Xing');
+        my $vbrScale;
         my $flags = unpack('x4N', $buff);
+        my $isVBR = ($buff !~ /^Info/);     # Info frame is not VBR (ref 5)
         $pos += 8;
         if ($flags & 0x01) {    # VBRFrames
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($vbrTable, 1, unpack("x${pos}N", $$buffPt));
+            $et->HandleTag($xingTable, 1, unpack("x${pos}N", $$buffPt)) if $isVBR;
             $pos += 4;
         }
         if ($flags & 0x02) {    # VBRBytes
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($vbrTable, 2, unpack("x${pos}N", $$buffPt));
+            $et->HandleTag($xingTable, 2, unpack("x${pos}N", $$buffPt)) if $isVBR;
             $pos += 4;
         }
         if ($flags & 0x04) {    # VBR_TOC
@@ -441,11 +528,54 @@ sub ParseMPEGAudio($$)
         }
         if ($flags & 0x08) {    # VBRScale
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($vbrTable, 3, unpack("x${pos}N", $$buffPt));
+            $vbrScale = unpack("x${pos}N", $$buffPt);
+            $et->HandleTag($xingTable, 3, $vbrScale) if $isVBR;
             $pos += 4;
         }
-        last;
-	}
+        # process Lame header (ref 5)
+        if ($flags & 0x10) {    # Lame
+            last if $pos + 348 > $len;
+        } elsif ($pos + 4 <= $len) {
+            my $lib = substr($$buffPt, $pos, 4);
+            unless ($lib eq 'LAME' or $lib eq 'GOGO') {
+                # attempt to identify other encoders
+                my $n;
+                if (index($$buffPt, 'RCA mp3PRO Encoder') >= 0) {
+                    $lib = 'RCA mp3PRO';
+                } elsif (($n = index($$buffPt, 'THOMSON mp3PRO Encoder')) >= 0) {
+                    $lib = 'Thomson mp3PRO';
+                    $n += 22;
+                    $lib .= ' ' . substr($$buffPt, $n, 6) if length($$buffPt) - $n >= 6;
+                } elsif (index($$buffPt, 'MPGE') >= 0) {
+                    $lib = 'Gogo (<3.0)';
+                } else {
+                    last;
+                }
+                $et->HandleTag($xingTable, 4, $lib);
+                last;
+            }
+        }
+        my $lameLen = $len - $pos;
+        last if $lameLen < 9;
+        my $enc = substr($$buffPt, $pos, 9);
+        if ($enc ge 'LAME3.90') {
+            $et->HandleTag($xingTable, 4, $enc);
+            if ($vbrScale <= 100) {
+                $et->HandleTag($xingTable, 5, int((100 - $vbrScale) / 10));
+                $et->HandleTag($xingTable, 6, (100 - $vbrScale) % 10);
+            }
+            my %dirInfo = (
+                DataPt   => $buffPt,
+                DirStart => $pos,
+                DirLen   => length($$buffPt) - $pos,
+            );
+            my $subTablePtr = GetTagTable('Image::ExifTool::MPEG::Lame');
+            $et->ProcessDirectory(\%dirInfo, $subTablePtr);
+        } else {
+            $et->HandleTag($xingTable, 4, substr($$buffPt, $pos, 20));
+        }
+        last;   # (didn't want to loop anyway)
+    }
 
     return 1;
 }
@@ -456,7 +586,7 @@ sub ParseMPEGAudio($$)
 # Returns: 1 on success, 0 if no video header was found
 sub ProcessMPEGVideo($$)
 {
-    my ($exifTool, $buffPt) = @_;
+    my ($et, $buffPt) = @_;
 
     return 0 unless length $$buffPt >= 4;
     my ($w1, $w2) = unpack('N2', $$buffPt);
@@ -469,10 +599,10 @@ sub ProcessMPEGVideo($$)
         return 0;
     }
     # set file type if not done already
-    $exifTool->SetFileType('MPEG') unless $exifTool->{VALUE}->{FileType};
+    $et->SetFileType('MPEG') unless $$et{VALUE}{FileType};
 
     my $tagTablePtr = GetTagTable('Image::ExifTool::MPEG::Video');
-    ProcessFrameHeader($exifTool, $tagTablePtr, $w1, $w2);
+    ProcessFrameHeader($et, $tagTablePtr, $w1, $w2);
     return 1;
 }
 
@@ -480,24 +610,38 @@ sub ProcessMPEGVideo($$)
 # Read MPEG audio and video frame headers
 # Inputs: 0) ExifTool object reference, 1) Reference to audio/video data
 # Returns: 1 on success, 0 if no video header was found
+# To Do: Properly parse MPEG streams:
+#   0xb7 - sequence end
+#   0xb9 - end code
+#   0xba - pack start code
+#   0xbb - system header
+#   0xbc - program map <-- should parse this
+#   0xbd - private stream 1 --> for VOB, this contains sub-streams:
+#           0x20-0x3f - pictures
+#           0x80-0x87 - audio (AC3,DTS,SDDS)
+#           0xa0-0xa7 - audio (LPCM)
+#   0xbe - padding
+#   0xbf - private stream 2
+#   0xc0-0xdf - audio stream
+#   0xe0-0xef - video stream
 sub ParseMPEGAudioVideo($$)
 {
-    my ($exifTool, $buffPt) = @_;
+    my ($et, $buffPt) = @_;
     my (%found, $didHdr);
     my $rtnVal = 0;
     my %proc = ( audio => \&ParseMPEGAudio, video => \&ProcessMPEGVideo );
 
-    delete $exifTool->{AudioBitrate};
-    delete $exifTool->{VideoBitrate};
+    delete $$et{AudioBitrate};
+    delete $$et{VideoBitrate};
 
     while ($$buffPt =~ /\0\0\x01(\xb3|\xc0)/g) {
         my $type = $1 eq "\xb3" ? 'video' : 'audio';
         unless ($didHdr) {
-            # make sure we didn't miss an audio frame sync before this (ie. MP3 file)
+            # make sure we didn't miss an audio frame sync before this (eg. MP3 file)
             # (the last byte of the 4-byte MP3 audio frame header word may be zero,
             # but the 2nd last must be non-zero, so we need only check to pos-3)
             my $buff = substr($$buffPt, 0, pos($$buffPt) - 3);
-            $found{audio} = 1 if ParseMPEGAudio($exifTool, \$buff);
+            $found{audio} = 1 if ParseMPEGAudio($et, \$buff);
             $didHdr = 1;
         }
         next if $found{$type};
@@ -506,7 +650,7 @@ sub ParseMPEGAudioVideo($$)
         $len > 256 and $len = 256;
         my $dat = substr($$buffPt, pos($$buffPt), $len);
         # process MPEG audio or video
-        if (&{$proc{$type}}($exifTool, \$dat)) {
+        if (&{$proc{$type}}($et, \$dat)) {
             $rtnVal = 1;
             $found{$type} = 1;
             # done if we found audio and video
@@ -522,18 +666,18 @@ sub ParseMPEGAudioVideo($$)
 # Returns: 1 on success, 0 if this wasn't a valid MPEG file
 sub ProcessMPEG($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $buff;
 
     $raf->Read($buff, 4) == 4 or return 0;
     return 0 unless $buff =~ /^\0\0\x01[\xb0-\xbf]/;
-    $exifTool->SetFileType();
+    $et->SetFileType();
 
     $raf->Seek(0,0);
     $raf->Read($buff, 65536*4) or return 0;
 
-    return ParseMPEGAudioVideo($exifTool, \$buff);
+    return ParseMPEGAudioVideo($et, \$buff);
 }
 
 1;  # end
@@ -560,7 +704,7 @@ based on unofficial sources which may be incomplete, inaccurate or outdated.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -573,6 +717,12 @@ under the same terms as Perl itself.
 
 =item L<http://www.getid3.org/>
 
+=item L<http://dvd.sourceforge.net/dvdinfo/dvdmpeg.html>
+
+=item L<http://ffmpeg.org/>
+
+=item L<http://sourceforge.net/projects/mediainfo/>
+
 =back
 
 =head1 SEE ALSO
