@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.41';
+$VERSION = '1.43';
 
 sub WritePS($$);
 sub ProcessPS($$;$);
@@ -89,14 +89,39 @@ sub ProcessPS($$;$);
         SubDirectory => {
             TagTable => 'Image::ExifTool::PostScript::Main',
         },
-        Notes => 'extracted with ExtractEmbedded option',
+        Notes => 'extracted with L<ExtractEmbedded|../ExifTool.html#ExtractEmbedded> option',
     },
     EmbeddedFileName => {
         Notes => q{
             not a real tag ID, but the file name from a BeginDocument statement.
-            Extracted with document metadata when ExtractEmbedded option is used
+            Extracted with document metadata when L<ExtractEmbedded|../ExifTool.html#ExtractEmbedded> option is used
         },
     },
+    # AI metadata (most with a single leading '%')
+    AI9_ColorModel => {
+        Name => 'AIColorModel',
+        PrintConv => {
+            1 => 'RGB',
+            2 => 'CMYK',
+        },
+    },
+    AI3_ColorUsage       => { Name => 'AIColorUsage' },
+    AI5_RulerUnits       => {
+        Name => 'AIRulerUnits',
+        PrintConv => {
+            0 => 'Inches',
+            1 => 'Millimeters',
+            2 => 'Points',
+            3 => 'Picas',
+            4 => 'Centimeters',
+            6 => 'Pixels',
+        },
+    },
+    AI5_TargetResolution => { Name => 'AITargetResolution' },
+    AI5_NumLayers        => { Name => 'AINumLayers' },
+    AI5_FileFormat       => { Name => 'AIFileFormat' },
+    AI8_CreatorVersion   => { Name => 'AICreatorVersion' }, # (double leading '%')
+    AI12_BuildNumber     => { Name => 'AIBuildNumber' },
 );
 
 # composite tags
@@ -183,9 +208,9 @@ sub GetInputRecordSeparator($)
     $a = pos($data), pos($data) = 0 if $data =~ /\x0a/g;
     $d = pos($data) if $data =~ /\x0d/g;
     my $diff = $a - $d;
-    if ($diff eq 1) {
+    if ($diff == 1) {
         $sep = "\x0d\x0a";
-    } elsif ($diff eq -1) {
+    } elsif ($diff == -1) {
         $sep = "\x0a\x0d";
     } elsif ($diff > 0) {
         $sep = "\x0d";
@@ -543,8 +568,8 @@ sub ProcessPS($$;$)
             next unless $data =~ m{<\?xpacket end=.(w|r).\?>($/|$)};
         } elsif ($data =~ /^%%?(\w+): ?(.*)/s and $$tagTablePtr{$1}) {
             my ($tag, $val) = ($1, $2);
-            # only allow 'ImageData' to have single leading '%'
-            next unless $data =~ /^%%/ or $1 eq 'ImageData';
+            # only allow 'ImageData' and AI tags to have single leading '%'
+            next unless $data =~ /^%(%|AI\d+_)/ or $tag eq 'ImageData';
             # decode comment string (reading continuation lines if necessary)
             $val = DecodeComment($val, $raf, \@lines);
             $et->HandleTag($tagTablePtr, $tag, $val);
@@ -674,7 +699,7 @@ This code reads meta information from EPS (Encapsulated PostScript), PS
 
 =head1 AUTHOR
 
-Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
