@@ -19,7 +19,7 @@ package Image::ExifTool::Shortcuts;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.51';
+$VERSION = '1.64';
 
 # this is a special table used to define command-line shortcuts
 # (documentation Notes may be added for these via %shortcutNotes in BuildTagLookup.pm)
@@ -117,6 +117,7 @@ $VERSION = '1.51';
         'MakerNoteCanon',
         'MakerNoteCasio',
         'MakerNoteCasio2',
+        'MakerNoteDJI',
         'MakerNoteFLIR',
         'MakerNoteFujiFilm',
         'MakerNoteGE',
@@ -143,11 +144,14 @@ $VERSION = '1.51';
         'MakerNoteKodak8c',
         'MakerNoteKodak9',
         'MakerNoteKodak10',
+        'MakerNoteKodak11',
+        'MakerNoteKodak12',
         'MakerNoteKodakUnknown',
         'MakerNoteKyocera',
         'MakerNoteMinolta',
         'MakerNoteMinolta2',
         'MakerNoteMinolta3',
+        'MakerNoteMotorola',
         'MakerNoteNikon',
         'MakerNoteNikon2',
         'MakerNoteNikon3',
@@ -160,8 +164,13 @@ $VERSION = '1.51';
         'MakerNoteLeica4',
         'MakerNoteLeica5',
         'MakerNoteLeica6',
+        'MakerNoteLeica7',
+        'MakerNoteLeica8',
+        'MakerNoteLeica9',
+        'MakerNoteLeica10',
         'MakerNotePanasonic',
         'MakerNotePanasonic2',
+        'MakerNotePanasonic3',
         'MakerNotePentax',
         'MakerNotePentax2',
         'MakerNotePentax3',
@@ -170,7 +179,11 @@ $VERSION = '1.51';
         'MakerNotePentax6',
         'MakerNotePhaseOne',
         'MakerNoteReconyx',
+        'MakerNoteReconyx2',
+        'MakerNoteReconyx3',
         'MakerNoteRicoh',
+        'MakerNoteRicoh2',
+        'MakerNoteRicohPentax',
         'MakerNoteRicohText',
         'MakerNoteSamsung1a',
         'MakerNoteSamsung1b',
@@ -179,6 +192,7 @@ $VERSION = '1.51';
         'MakerNoteSanyoC4',
         'MakerNoteSanyoPatch',
         'MakerNoteSigma',
+        'MakerNoteSigma3',
         'MakerNoteSony',
         'MakerNoteSony2',
         'MakerNoteSony3',
@@ -222,7 +236,7 @@ $VERSION = '1.51';
         'IFD0:Artist',
         'IFD0:Copyright',
         # other TIFF tags
-        'IFD0:Rating', 
+        'IFD0:Rating',
         'IFD0:RatingPercent',
         'IFD0:DNGLensInfo',
         'IFD0:PanasonicTitle',
@@ -233,6 +247,25 @@ $VERSION = '1.51';
         'IFD0:XPKeywords',
         'IFD0:XPSubject',
     ],
+    # large binary data tags which won't be loaded if excluded when extracting
+    LargeTags => [
+        'CanonVRD',
+        'DLOData',
+        'EXIF',
+        'ICC_Profile',
+        'IDCPreviewImage',
+        'ImageData',
+        'IPTC',
+        'JpgFromRaw',
+        'OriginalRawImage',
+        'OtherImage',
+        'PreviewImage',
+        'ThumbnailImage',
+        'TIFFPreview',
+        'XML',
+        'XMP',
+        'ZoomedPreviewImage',
+    ],
 );
 
 #------------------------------------------------------------------------------
@@ -300,7 +333,7 @@ name prefix.
 
 =head1 AUTHOR
 
-Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
